@@ -1,42 +1,25 @@
 import copy
 import logging
 from omgeo.places import PlaceQuery
-from omgeo.processors.postprocessors import DupePicker, SnapPoints
+from omgeo.postprocessors import DupePicker, SnapPoints
 import time
 
 stats_logger = logging.getLogger('omgeo.stats')
 
 class Geocoder():
     """
-    17h11
-    The base geocode class.  This class can be initialized with settings
-    for each geocoder and/or settings for the geocoder itself.
-
-    Arguments:
-    ==========
-    sources         -- a dictionary of GeocodeServiceConfig() parameters,
-                       keyed by module name for the GeocodeService to use
-                       ex: {'esri_na':{}, 
-                            'bing': {'settings': {},
-                                     'preprocessors': [],
-                                     'postprocessors': []},
-                            ...}
-    preprocessors   -- list of universal preprocessors to use
-    postprocessors  -- list of universal postprocessors to use
-    waterfall       -- sets default for waterfall on geocode() method
-                       (default False)
+    Class for building a custom geocoder using external APIs.
     """
 
-    DEFAULT_SOURCES = [['omgeo.services.EsriNA', {}],
-                       ['omgeo.services.EsriEU', {}],
-                       ['omgeo.services.Nominatim', {}]]
+    DEFAULT_SOURCES = [['omgeo.services.EsriWGS', {}],
+                       ['omgeo.services.Nominatim', {}]
+                      ]
     DEFAULT_PREPROCESSORS = []
-    DEFAULT_POSTPROCESSORS = [SnapPoints(),
-                              DupePicker('match_addr', 'locator',
-                                         ['rooftop',
-                                          'parcel',
-                                          'interpolation_offset',
-                                          'interpolation'])]
+    DEFAULT_POSTPROCESSORS = [
+        SnapPoints(),
+        DupePicker('match_addr', 'locator',
+                   ['rooftop', 'parcel', 'interpolation_offset', 'interpolation'])
+            ]
     
     def _get_service_by_name(self, service_name):
         try:
@@ -60,11 +43,6 @@ class Geocoder():
     def set_sources(self, sources):
         """
         Creates GeocodeServiceConfigs from each str source
-        
-        Argument:
-        =========
-        sources --  list of source-settings pairs
-                    ex. "[['EsriNA', {}], ['Nominatim', {}]]"
         """
         if len(sources) == 0:
             raise Exception('Must declare at least one source for a geocoder')
@@ -74,6 +52,21 @@ class Geocoder():
 
     def __init__(self, sources=None, preprocessors=None, postprocessors=None,
                  waterfall=False):
+        """
+        :arg dict sources: a dictionary of GeocodeServiceConfig() parameters,
+                           keyed by module name for the GeocodeService to use, e.g.::
+
+                               {'esri_wgs':{}, 
+                                'bing': {'settings': {},
+                                         'preprocessors': [],
+                                         'postprocessors': []},
+                                 ...}
+
+        :arg list preprocessors: list of universal preprocessors to use
+        :arg list postprocessors: list of universal postprocessors to use
+        :arg bool waterfall: sets default ``for waterfall on geocode() method (default ``False````)
+        """
+
         self._preprocessors = Geocoder.DEFAULT_PREPROCESSORS \
             if preprocessors is None else preprocessors
         self._postprocessors = Geocoder.DEFAULT_POSTPROCESSORS \
@@ -84,18 +77,16 @@ class Geocoder():
         
     def geocode(self, pq, waterfall=None):
         """
-        Returns a dictionary including:
-         * candidates - list of Candidate objects
-         * upstream_response_info - list of UpstreamResponseInfo objects
+        :arg PlaceQuery pq:  PlaceQuery object (required).
+        :arg bool waterfall: Boolean set to True if all geocoders listed should
+                             be used to find results, instead of stopping after
+                             the first geocoding service with valid candidates
+                             (defaults to self.waterfall).
+        :returns: Returns a dictionary including:
+                   * candidates - list of Candidate objects
+                   * upstream_response_info - list of UpstreamResponseInfo objects
+        """      
 
-        Arguments:
-        ==========
-        pq          --  A PlaceQuery object (required).
-        waterfall   --  Boolean set to True if all geocoders listed should
-                        be used to find results, instead of stopping after
-                        the first geocoding service with valid candidates
-                        (defaults to <Geocoder instance>.waterfall).
-        """          
         start_time = time.time()
         waterfall = self.waterfall if waterfall is None else waterfall
         if type(pq) in (str, unicode):
@@ -139,5 +130,4 @@ class Geocoder():
             return uri_dict
         uri_set = [get_uri_dict(uri_item) for uri_item in result['upstream_response_info']]
         return dict(candidates=[candidate.__dict__ for candidate in result['candidates']],
-                    upstream_response_info=uri_set)
-        
\ No newline at end of file
+                    upstream_response_info=uri_set)
\ No newline at end of file
