@@ -2,8 +2,10 @@ from base import GeocodeService
 import json
 import logging
 from omgeo.places import Candidate
-from omgeo.processors.preprocessors import CountryPreProcessor, RequireCountry, ParseSingleLine, ReplaceRangeWithNumber
-from omgeo.processors.postprocessors import AttrFilter, AttrExclude, AttrRename, AttrSorter, AttrMigrator, UseHighScoreIfAtLeast, GroupBy, ScoreSorter
+from omgeo.preprocessors import CancelIfPOBox, CountryPreProcessor, RequireCountry, \
+    ParseSingleLine, ReplaceRangeWithNumber
+from omgeo.postprocessors import AttrFilter, AttrExclude, AttrRename, AttrSorter, \
+    AttrMigrator, UseHighScoreIfAtLeast, GroupBy, ScoreSorter
 from suds.client import Client
 import time
 from urllib import unquote
@@ -17,8 +19,8 @@ class Bing(GeocodeService):
      * `Find a Location by Address <http://msdn.microsoft.com/en-us/library/ff701714.aspx>`_
 
     Settings used by the Bing GeocodeService object may include:
-    ============================================================
-    api_key --  The API key used to access Bing services.
+     * api_key --  The API key used to access Bing services.
+
     """
     _endpoint = 'http://dev.virtualearth.net/REST/v1/Locations'
 
@@ -178,26 +180,22 @@ class CitizenAtlas(GeocodeService):
 
 class EsriGeocodeService(GeocodeService):
     """
-    Settings used by an EsriGeocodeService object may include:
-    ============================================================
-    api_key --  The API key used to access ESRI premium services.  If this
-                key is present, the object's endpoint will be set to use
-                premium tasks.
+    Base class for older ESRI geocoders (EsriEU, EsriNA).
     """
 
     def __init__(self, preprocessors=None, postprocessors=None, settings=None):
         """
-        ESRI services can be used as free services or "premium tasks".  If an
-        ESRI service is created with an api_key in the settings, we'll set this
-        service up with the premium task URL.
+        :arg list preprocessors: preprocessors
+        :arg list postprocessors: postprocessors
+        :arg dict settings: Settings used by an EsriGeocodeService object may include
+                            the ``api_key`` used to access ESRI premium services.  
+                            If this key is present, the object's endpoint will be
+                            set to use premium tasks.
+
         """
         GeocodeService.__init__(self, preprocessors, postprocessors, settings)
-        
-        if 'api_key' in self._settings:
-            service_url = 'http://premiumtasks.arcgisonline.com/server'
-        else:
-            service_url = 'http://tasks.arcgisonline.com/ArcGIS'
-
+        service_url = 'http://premiumtasks.arcgisonline.com/server' if 'api_key' in self._settings \
+            else 'http://tasks.arcgisonline.com/ArcGIS'
         self._endpoint = service_url + self._task_endpoint
 
     def append_token_if_needed(self, query_dict):
@@ -265,7 +263,7 @@ class EsriSoapGeocodeService(EsriGeocodeService):
 
 class EsriEUGeocodeService():
     """
-    Defaults for Esri EU Geocoders
+    Base class including for Esri EU REST and SOAP Geocoders
 
     As of 29 Dec 2011, the ESRI website claims to support Andorra, Austria, 
     Belgium, Denmark, Finland, France, Germany, Gibraltar, Ireland, Italy,
@@ -274,14 +272,15 @@ class EsriEUGeocodeService():
     """
     _wkid = 4326
 
+    #: FIPS codes of supported countries
     SUPPORTED_COUNTRIES_FIPS = ['AN', 'AU', 'BE', 'DA', 'FI', 'FR', 'GM', 'GI', 'EI', 
         'IT', 'LS', 'LU', 'MN', 'NL', 'NO', 'PO', 'SM', 'SP', 'SW', 'SZ', 'UK', 'VT']
-    """FIPS codes of supported countries"""
 
+    #: ISO-2 codes of supported countries
     SUPPORTED_COUNTRIES_ISO2 = ['AD', 'AT', 'BE', 'DK', 'FI', 'FR', 'DE', 'GI', 'IE', 
         'IT', 'LI', 'LU', 'MC', 'NL', 'NO', 'PT', 'SM', 'ES', 'SE', 'CH', 'GB', 'VC']
-    """ISO-2 codes of supported countries"""
 
+    #: Map of FIPS to ISO-2 codes, if they are different.
     MAP_FIPS_TO_ISO2 = {
         'AN':'AD',
         'AU':'AT',
@@ -297,8 +296,8 @@ class EsriEUGeocodeService():
         'UK':'GB',
         'VT':'VC',
         }
-    """Map of FIPS to ISO-2 codes, if they are different."""
 
+    #: Map to standardize locator
     LOCATOR_MAP = {
         'EU_Street_Addr': 'interpolation',
     }
@@ -308,7 +307,6 @@ class EsriEUGeocodeService():
             SUPPORTED_COUNTRIES_ISO2,
             MAP_FIPS_TO_ISO2),
         ParseSingleLine(),
-        RequireCountry('GB'),
     ]
         
     DEFAULT_POSTPROCESSORS = [
@@ -480,8 +478,7 @@ class EsriNASoap(EsriSoapGeocodeService, EsriNAGeocodeService):
                 return []
 
         return candidates
-        
-        
+         
 class EsriNA(EsriGeocodeService, EsriNAGeocodeService):
     """Esri REST Geocoder for North America"""
     _task_endpoint = '/rest/services/Locators/TA_Address_NA_10/GeocodeServer/findAddressCandidates'
@@ -495,7 +492,6 @@ class EsriNA(EsriGeocodeService, EsriNAGeocodeService):
         
         EsriGeocodeService.__init__(self, preprocessors, postprocessors, settings)
 
-
     def _geocode(self, location):
         query = {
             'SingleLine':location.query,
@@ -531,6 +527,154 @@ class EsriNA(EsriGeocodeService, EsriNAGeocodeService):
             pass
         return returned_candidates
     
+class EsriWGS(GeocodeService):
+    """
+    Class to geocode using the ESRI World Geocoding service
+    <http://geocode.arcgis.com/arcgis/geocoding.html>.
+
+    This uses two endpoints -- one for single-line addresses,
+    and one for multi-part addresses.
+    """
+
+    LOCATOR_MAP = {
+        'PointAddress': 'rooftop',
+        'StreetAddress': 'interpolation',
+        'PostalExt': 'postal_specific', # accept ZIP+4
+        'Postal': 'postal'
+    }
+
+    DEFAULT_PREPROCESSORS = [CancelIfPOBox()]
+
+    DEFAULT_POSTPROCESSORS = [
+        AttrFilter(['PointAddress',
+                    'StreetAddress',
+                    #'PostalExt',
+                    #'Postal'
+                   ],
+                   'locator_type'),
+        #AttrExclude(['USA_Postal'], 'locator'), #accept postal from everywhere but US (need PostalExt)
+        AttrSorter(['PointAddress',
+                    'StreetAddress',
+                    #'PostalExt',
+                    #'Postal'
+                   ],
+                   'locator_type'),
+        AttrRename('locator', LOCATOR_MAP), # after filter to avoid searching things we toss out
+        UseHighScoreIfAtLeast(99.8),
+        GroupBy('match_addr'),
+        GroupBy(('x', 'y')),
+        ScoreSorter(),
+    ]
+
+    _endpoint = 'http://geocode.arcgis.com/arcgis/rest/services/World/GeocodeServer'
+
+    def _geocode(self, pq):
+        """
+        :arg PlaceQuery pq: PlaceQuery object to use for geocoding
+        :returns: list of location Candidates
+        """
+        #: List of desired output fields
+        #: See `ESRI docs <http://geocode.arcgis.com/arcgis/geocoding.html#output>_` for details
+        outFields = ('Loc_name',
+                     #'Shape',
+                     'Score',
+                     #'Match_Addr', #based on address standards for the country
+                     #'Address', # returned by default
+                     #'Country' # 3-digit ISO 3166-1 code for a country. Example: Canada = "CAN"
+                     #'Admin',
+                     #'DepAdmin',
+                     #'SubAdmin',
+                     #'Locality',
+                     #'Postal',
+                     #'PostalExt',
+                     'Addr_Type',
+                     #'Type',
+                     #'Rank',
+                     #'AddNum',
+                     #'StPreDir',
+                     #'StPreType',
+                     #'StName',
+                     #'StType',
+                     #'StDir',
+                     #'Side',
+                     #'AddNumFrom',
+                     #'AddNumTo',
+                     #'AddBldg',
+                     #'Ymax',
+                     #'Ymin',
+                     #'Xmin',
+                     #'Xmax',
+                     #'X',
+                     #'Y',
+                     'DisplayX',
+                     'DisplayY',
+                     #'LangCode',
+                     #'Status',
+                    )
+        outFields = ','.join(outFields)
+        query = dict(f='json', # default HTML. Other options are JSON and KMZ.
+                     outFields=outFields,
+                     #outSR=WKID, defaults to 4326
+                     maxLocations=20, # default 1; max is 20
+                     )
+
+        if pq.query == '': # multipart
+            method = 'findAddressCandidates'
+            query = dict(query,
+                         Address=pq.address, # commonly represents the house number and street name of a complete address
+                         Admin1=pq.city,
+                         Admin2=pq.state,
+                         #Admin3=
+                         #Admin4=
+                         Postal=pq.postal,
+                         #PostalExt=
+                         CountryCode=pq.country, # full country name or ISO 3166-1 2- or 3-digit country code
+                         )
+            if pq.bounded and pq.viewbox is not None:
+                query = dict(query, searchExtent=pq.viewbox.to_esri_wgs_json())            
+        else: # single-line
+            method = 'find'
+            query = dict(query,
+                         text=pq.query, # This can be a street address, place name, postal code, or POI.
+                         sourceCountry=pq.country, # full country name or ISO 3166-1 2- or 3-digit country code
+                         )
+            if pq.bounded and pq.viewbox is not None:
+                query = dict(query, bbox=pq.viewbox.to_esri_wgs_json())
+
+        endpoint = self._endpoint + '/' + method
+        response_obj = self._get_json_obj(endpoint, query)
+        returned_candidates = [] # this will be the list returned
+        try: 
+            for location in response_obj['locations']:         
+                c = Candidate()
+                attributes = location['feature']['attributes']
+                c.locator = attributes['Loc_name']
+                c.locator_type = attributes['Addr_Type']
+                c.score = attributes['Score']
+                c.match_addr = location['name']
+                #: "represents the actual location of the address. It differs from the X value"
+                c.x = attributes['DisplayX'] 
+                #: "represents the actual location of the address. It differs from the Y value"
+                c.y = attributes['DisplayY']
+                c.wkid = response_obj['spatialReference']['wkid']
+                c.geoservice = self.__class__.__name__
+                returned_candidates.append(c)
+        except KeyError:
+            pass
+        return returned_candidates   
+
+    def __init__(self, preprocessors=None, postprocessors=None, settings=None):
+        preprocessors = EsriWGS.DEFAULT_PREPROCESSORS if preprocessors is None else preprocessors
+        postprocessors = EsriWGS.DEFAULT_POSTPROCESSORS if postprocessors is None else postprocessors
+        GeocodeService.__init__(self, preprocessors, postprocessors, settings)
+
+class EsriWGSSSL(EsriWGS):
+    """ 
+    Class to geocode using the ESRI World Geocoding service over SSL
+    <https://geocode.arcgis.com/arcgis/geocoding.html>
+    """
+    _endpoint = 'https://geocode.arcgis.com/arcgis/rest/services/World/GeocodeServer'
+
 class MapQuest(GeocodeService):
     """
     Class to geocode using MapQuest licensed services.
@@ -608,8 +752,8 @@ class Nominatim(GeocodeService):
     """Postprocessors to use with this geocoder service, in order of desired execution."""
     
     def __init__(self, preprocessors=None, postprocessors=None, settings=None):
-        preprocessors = Bing.DEFAULT_PREPROCESSORS if preprocessors is None else preprocessors
-        postprocessors = Bing.DEFAULT_POSTPROCESSORS if postprocessors is None else postprocessors
+        preprocessors = Nominatim.DEFAULT_PREPROCESSORS if preprocessors is None else preprocessors
+        postprocessors = Nominatim.DEFAULT_POSTPROCESSORS if postprocessors is None else postprocessors
         GeocodeService.__init__(self, preprocessors, postprocessors, settings)
 
     def _geocode(self, pq):
@@ -633,4 +777,4 @@ class Nominatim(GeocodeService):
             c.wkid = self._wkid
             c.geoservice = self.__class__.__name__
             returned_candidates.append(c)
-        return returned_candidates
+        return returned_candidates
\ No newline at end of file
